'use strict';

var _ErrorHelper = require('../utils/ErrorHelper');

var _ErrorHelper2 = _interopRequireDefault(_ErrorHelper);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } } /** @module */

/**
 * 转换为二维数组
 * @param spliceNum 每个小数组几个元素
 * @param arrayNum 分成几个小数组
 * @returns {*}
 */
var toTwoDimen = function toTwoDimen() {
  var spliceNum = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
  var arrayNum = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

  if (spliceNum !== null && arrayNum === null) {
    var num = this.length % spliceNum === 0 ? parseInt(this.length / spliceNum) : parseInt(this.length / spliceNum) + 1;
    var newArrays = [];
    for (var i = 0; i < num; i++) {
      var arr = this.slice(spliceNum * i, spliceNum * (i + 1));
      if (arr.length > 0) {
        newArrays.push(arr);
      }
    }
    return newArrays;
  } else if (spliceNum === null && arrayNum !== null) {
    var _newArrays = [];
    var _num = parseInt(this.length / arrayNum);
    for (var _i = 0; _i < arrayNum; _i++) {
      var _arr = this.slice(_num * _i, _num * (_i + 1));
      if (_arr.length > 0) {
        _newArrays.push(_arr);
      }
    }
    if (_num * arrayNum < this.length) {
      _newArrays[_newArrays.length - 1] = _newArrays[_newArrays.length - 1].concat(this.slice(_num * arrayNum, this.length));
    }
    return _newArrays;
  } else {
    return null;
  }
};

/**
 * 简单去重
 */
var uniq = function uniq() {
  return [].concat(_toConsumableArray(new Set(this)));
};

/**
 * 移除null、undefined以及空字符串
 * @returns {Array}
 */
var removeEmpty = function removeEmpty() {
  var results = [];
  this.forEach(function (ele) {
    if (ele !== null && ele !== undefined && ele !== '') {
      results.push(ele);
    }
  });
  return results;
};

/**
 * 计算平均数
 * @returns {any}
 */
var getAverage = function getAverage() {
  var sum = '0';
  this.forEach(function (ele) {
    sum = sum.add(ele);
  });
  return sum.div(this.length.toString());
};

/**
 * 取数组最后一个元素
 * @returns {*}
 */
var getLastOne = function getLastOne() {
  return this[this.length - 1];
};

var removeLastOne = function removeLastOne() {
  if (this.length === 0) {
    return [];
  }
  return this.slice(0, this.length - 1);
};

/**
 * 取数组第一个元素
 * @returns {*}
 */
var getFirstOne = function getFirstOne() {
  return this[0];
};

var removeFirstOne = function removeFirstOne() {
  if (this.length === 0) {
    return [];
  }
  return this.slice(1, this.length);
};

var removeStart = function removeStart(num) {
  return this.slice(num, this.length);
};

var removeEnd = function removeEnd(num) {
  return this.slice(0, this.length - num);
};

var removeByIndex = function removeByIndex(index) {
  if (index > this.length - 1) {
    throw (0, _ErrorHelper2.default)('\u7D22\u5F15\u8D85\u8FC7\u6570\u7EC4\u957F\u5EA6');
  }
  var temp = this.deepCopy();
  temp.splice(index, 1);
  return temp;
};

var removeByValue = function removeByValue(value) {
  var temp = this.deepCopy();
  var index = temp.indexOf(value);
  if (index === -1) {
    throw (0, _ErrorHelper2.default)('\u6CA1\u6709\u627E\u5230 ' + value);
  }
  temp.splice(index, 1);
  return temp;
};

var deepCopy = function deepCopy() {
  return Array.from(this);
};

var append = function append(arr) {
  return this.concat(arr);
};

/**
 * 取出最大值（值以及索引）,只适用于数值数组以及字符串数组, 返回值为字符串
 */
var getMax = function getMax() {
  if (this.length === 0) {
    throw (0, _ErrorHelper2.default)('\u7A7A\u6570\u7EC4');
  }

  var maxValue = this[0].toString();

  for (var i = 1; i < this.length; i++) {
    if (this[i].toString().gt(maxValue)) {
      maxValue = this[i].toString();
    }
  }

  var maxIndex = [];
  for (var _i2 = 0; _i2 < this.length; _i2++) {
    if (this[_i2].toString().eq(maxValue)) {
      maxIndex.push(_i2);
    }
  }

  return {
    value: maxValue,
    index: maxIndex
  };
};

var getMin = function getMin() {
  if (this.length === 0) {
    throw (0, _ErrorHelper2.default)('\u7A7A\u6570\u7EC4');
  }

  var minValue = this[0].toString();

  for (var i = 1; i < this.length; i++) {
    if (this[i].toString().lt(minValue)) {
      minValue = this[i].toString();
    }
  }

  var minIndex = [];
  for (var _i3 = 0; _i3 < this.length; _i3++) {
    if (this[_i3].toString().eq(minValue)) {
      minIndex.push(_i3);
    }
  }

  return {
    value: minValue,
    index: minIndex
  };
};

/**
 * 取出最大值（值以及索引）,只适用于数值数组以及字符串数组, 返回值为字符串
 */
var getMaxV2 = function getMaxV2() {
  if (this.length === 0) {
    throw (0, _ErrorHelper2.default)('\u7A7A\u6570\u7EC4');
  }

  var maxValue = this[0].toString();

  for (var i = 1; i < this.length; i++) {
    if (this[i].toString().gt(maxValue)) {
      maxValue = this[i].toString();
    }
  }

  var maxIndex = [];
  for (var _i4 = 0; _i4 < this.length; _i4++) {
    if (this[_i4].toString().eq(maxValue)) {
      maxIndex.push(_i4);
    }
  }

  return {
    value: maxValue,
    indexes: maxIndex
  };
};

var getMinV2 = function getMinV2() {
  if (this.length === 0) {
    throw (0, _ErrorHelper2.default)('\u7A7A\u6570\u7EC4');
  }

  var minValue = this[0].toString();

  for (var i = 1; i < this.length; i++) {
    if (this[i].toString().lt(minValue)) {
      minValue = this[i].toString();
    }
  }

  var minIndex = [];
  for (var _i5 = 0; _i5 < this.length; _i5++) {
    if (this[_i5].toString().eq(minValue)) {
      minIndex.push(_i5);
    }
  }

  return {
    value: minValue,
    indexes: minIndex
  };
};

var getSum = function getSum() {
  return this.reduce(function (acc, val) {
    return val.toString().add(acc);
  });
};

var select = function select(indexes) {
  var _this = this;

  if (indexes === null || indexes === undefined) {
    return this;
  }
  var result = [];
  indexes.forEach(function (index) {
    result.push(_this[index]);
  });
  return result;
};

var toUpperCase = function toUpperCase() {
  return this.map(function (element) {
    return element.toString().toUpperCase();
  });
};

var toLowerCase = function toLowerCase() {
  return this.map(function (element) {
    return element.toString().toLowerCase();
  });
};

var random = function random() {
  if (this.length === 0) {
    throw (0, _ErrorHelper2.default)('\u7A7A\u6570\u7EC4');
  }
  return this[Math.floor(Math.random() * 1E4) % this.length];
};

/**
 * [190, 190] --> BEBE
 * @returns {string}
 */
var numberArrayToHexString = function numberArrayToHexString() {
  if (this.length === 0) {
    throw (0, _ErrorHelper2.default)('\u7A7A\u6570\u7EC4');
  }
  return this.map(function (byte) {
    return byte.toString().numberStrToHex();
  }).join('');
};

/**
 * 取子数组。[start, end)
 * @param start
 * @param end
 */
var sub = function sub(start, end) {
  if (this.length === 0) {
    throw (0, _ErrorHelper2.default)('\u7A7A\u6570\u7EC4');
  }
  return this.slice(start, end);
};

Array.prototype.toTwoDimen = toTwoDimen;
Array.prototype.uniq = uniq;
Array.prototype.removeEmpty = removeEmpty;
Array.prototype.getAverage = getAverage;
Array.prototype.getLastOne = getLastOne;
Array.prototype.getFirstOne = getFirstOne;
Array.prototype.removeFirstOne = removeFirstOne;
Array.prototype.removeLastOne = removeLastOne;
Array.prototype.removeStart = removeStart;
Array.prototype.removeEnd = removeEnd;
Array.prototype.removeByIndex = removeByIndex;
Array.prototype.deepCopy = deepCopy;
Array.prototype.removeByValue = removeByValue;
Array.prototype.append = append;
Array.prototype.getMax = getMax;
Array.prototype.getMin = getMin;
Array.prototype.getMaxV2 = getMaxV2;
Array.prototype.getMinV2 = getMinV2;
Array.prototype.getSum = getSum;
Array.prototype.select = select;
Array.prototype.toUpperCase = toUpperCase;
Array.prototype.toLowerCase = toLowerCase;
Array.prototype.random = random;
Array.prototype.numberArrayToHexString = numberArrayToHexString;
Array.prototype.sub = sub;