'use strict';

var _bignumber = require('bignumber.js');

var _bignumber2 = _interopRequireDefault(_bignumber);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 转换为十进制的bytes数组
 * @returns {string}
 */
var toDecimalNumberArray = function toDecimalNumberArray() {
  var temp = this.correctEmptyBuffer();
  return [].slice.call(temp);
};

/**
 * 转化为二进制字符串
 * @returns {string}
 */
/** @module */

var toBinString = function toBinString() {
  var temp = this;
  return temp.toDecimalArray().map(function (x) {
    var str = x.toString(2);
    while (str.length < 8) {
      str = '0' + str;
    }
    return str;
  }).join('');
};

/**
 * 转化为十六进制字符串
 * @returns {string}
 */
var toHexString = function toHexString() {
  var prefix = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

  var temp = this.toString('hex');
  if (prefix === false) {
    return temp;
  } else {
    return '0x' + temp;
  }
};

/**
 * 转化为十进制字符串
 * @returns {string}
 */
var toDecimalString = function toDecimalString() {
  if (this.getBytesLength() === 0) {
    return '0';
  }
  return new _bignumber2.default(this.toString('hex'), 16).toString();
};

/**
 * 转化为十进制数值
 * @returns {number}
 */
var toDecimalNumber = function toDecimalNumber() {
  var temp = this.correctEmptyBuffer();
  return new _bignumber2.default(temp.toString('hex'), 16).toNumber();
};

/**
 * 转化为十六进制数值
 * @returns {number | *}
 */
var toHexNumber = function toHexNumber() {
  var temp = this.correctEmptyBuffer();
  return temp.toHexString(false).toNumber();
};

/**
 * 纠正空buffer
 * @returns {correctEmptyBuffer}
 */
var correctEmptyBuffer = function correctEmptyBuffer() {
  var temp = this;
  if (this.getBytesLength() === 0) {
    temp = new Buffer('00', 'hex');
  }
  return temp;
};

/**
 * 获取字节数
 */
var getBytesLength = function getBytesLength() {
  return this.length;
};

/**
 * 反转Buffer, 不改变自身
 * @returns {Buffer}
 */
var reverseBuffer = function reverseBuffer() {
  return this.deepCopy().correctEmptyBuffer().reverse();
};

/**
 * 深拷贝
 * @returns {*}
 */
var deepCopy = function deepCopy() {
  var tempBuffer = Buffer.alloc(this.length);
  this.copy(tempBuffer, 0, 0, this.length);
  return tempBuffer;
};

Buffer.prototype.toDecimalNumberArray = toDecimalNumberArray;
Buffer.prototype.toBinString = toBinString;
Buffer.prototype.toHexString = toHexString;
Buffer.prototype.toDecimalNumber = toDecimalNumber;
Buffer.prototype.getBytesLength = getBytesLength;
Buffer.prototype.correctEmptyBuffer = correctEmptyBuffer;
Buffer.prototype.reverseBuffer = reverseBuffer;
Buffer.prototype.toHexNumber = toHexNumber;
Buffer.prototype.toDecimalString = toDecimalString;
Buffer.prototype.deepCopy = deepCopy;