'use strict';

var _bignumber = require('bignumber.js');

var _bignumber2 = _interopRequireDefault(_bignumber);

var _AssertUtil = require('../utils/AssertUtil');

var _AssertUtil2 = _interopRequireDefault(_AssertUtil);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// 进制计算的结果都要带上相应前缀 二进制0b 八进制0o 十六进制0x

/**
 * 加
 * @param val
 * @returns {any}
 */
/** @module */

var add = function add(val) {
  _AssertUtil2.default.canCast(this, 'bignumber');
  val = val.toString();
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num1 = new BN(this);
  return num1.plus(val).toString();
};

/**
 * 乘以10的几次方
 * @param num
 */
var shiftedBy = function shiftedBy(num) {
  _AssertUtil2.default.canCast(this, 'bignumber');
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num1 = new BN(this);
  return num1.shiftedBy(num.toNumber()).toString();
};

var unShiftedBy = function unShiftedBy(num) {
  _AssertUtil2.default.canCast(this, 'bignumber');
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num1 = new BN(this);
  return num1.shiftedBy(num.toString().negated().toNumber()).toString();
};

var negated = function negated() {
  _AssertUtil2.default.canCast(this, 'bignumber');
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num1 = new BN(this);
  return num1.negated().toString();
};

/**
 * 减
 * @param val
 * @returns {any}
 */
var sub = function sub(val) {
  _AssertUtil2.default.canCast(this, 'bignumber');
  val = val.toString();
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num1 = new BN(this);
  return num1.minus(val).toString();
};

/**
 * 乘
 * @param val
 * @returns {any}
 */
var multi = function multi(val) {
  _AssertUtil2.default.canCast(this, 'bignumber');
  val = val.toString();
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num1 = new BN(this);
  return num1.times(val).toString();
};

/**
 * 乘方
 * @param val
 * @returns {any}
 */
var pow = function pow(val) {
  _AssertUtil2.default.canCast(this, 'bignumber');
  val = val.toNumber();
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num1 = new BN(this);
  return num1.exponentiatedBy(val).toString();
};

/**
 * 除
 * @param val
 * @returns {string}
 */
var div = function div(val) {
  _AssertUtil2.default.canCast(this, 'bignumber');
  val = val.toString();
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num1 = new BN(this);
  return num1.div(val).toString();
};

/**
 * 求余
 * @param val
 * @returns {string}
 */
var mod = function mod(val) {
  _AssertUtil2.default.canCast(this, 'bignumber');
  val = val.toString();
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num1 = new BN(this);
  return num1.mod(val).toString();
};

/**
 * 开根号
 * @returns {string}
 */
var sqrt = function sqrt() {
  _AssertUtil2.default.canCast(this, 'bignumber');
  val = val.toString();
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num1 = new BN(this);
  return num1.sqrt().toString();
};

/**
 * 是否大于
 * @param val
 * @returns {boolean}
 */
var gt = function gt(val) {
  _AssertUtil2.default.canCast(this, 'bignumber');
  val = val.toString();
  var num1 = new _bignumber2.default(this);
  return num1.comparedTo(val) === 1;
};

/**
 * 大于或等于
 * @param val
 * @returns {boolean}
 */
var gtOrEq = function gtOrEq(val) {
  _AssertUtil2.default.canCast(this, 'bignumber');
  val = val.toString();
  var num1 = new _bignumber2.default(this);
  var result = num1.comparedTo(val);
  return result === 1 || result === 0;
};

/**
 * 是否小于
 * @param val
 * @returns {boolean}
 */
var lt = function lt(val) {
  _AssertUtil2.default.canCast(this, 'bignumber');
  val = val.toString();
  var num1 = new _bignumber2.default(this);
  return num1.comparedTo(val) === -1;
};

/**
 * 小于或等于
 * @param val
 * @returns {boolean}
 */
var ltOrEq = function ltOrEq(val) {
  _AssertUtil2.default.canCast(this, 'bignumber');
  val = val.toString();
  var num1 = new _bignumber2.default(this);
  var result = num1.comparedTo(val);
  return result === -1 || result === 0;
};

/**
 * 是否相等
 * @param val
 * @returns {boolean}
 */
var eq = function eq(val) {
  _AssertUtil2.default.canCast(this, 'bignumber');
  val = val.toString();
  var num1 = new _bignumber2.default(this);
  return num1.comparedTo(val) === 0;
};

/**
 * 加千分号
 * @returns {string}
 */
var addThousandSign = function addThousandSign() {
  _AssertUtil2.default.canCast(this, 'bignumber');
  var parts = this.split('.');
  parts[0] = parts[0].replace(/\B(?=(\d{3})+(?!\d))/g, ',');
  return parts[1] === '' ? parts[0] : parts.join('.');
};

/**
 * 移除千分号
 * @returns {string}
 */
var removeThousandSign = function removeThousandSign() {
  _AssertUtil2.default.canCast(this, 'bignumber');
  return this.replace(new RegExp(',', 'g'), '');
};

/**
 * 保留小数点后几位
 * @param decimalRemain
 * @param remainMethod {number} ROUND_UP 0(直接截断，向上取整), ROUND_DOWN 1(直接截断，向下取整), ROUND_CEIL 2, ROUND_FLOOR 3, ROUND_HALF_UP 4(遇到.5的情况时往上近似,就是四舍五入), ROUND_HALF_DOWN 5(遇到.5的情况时往下近似), ROUND_HALF_EVEN 6, ROUND_HALF_CEIL 7, ROUND_HALF_FLOOR 8, EUCLID 9
 * @returns {string}
 */
var remainDecimal = function remainDecimal() {
  var decimalRemain = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
  var remainMethod = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _bignumber2.default.ROUND_HALF_UP;

  _AssertUtil2.default.canCast(this, 'bignumber');
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num1 = new BN(this);
  return num1.toFixed(decimalRemain, remainMethod);
};

/**
 * 获取小数部分的个数
 */
var decimalCount = function decimalCount() {
  _AssertUtil2.default.canCast(this, 'bignumber');
  return this.split('.')[1].length;
};

/**
 * 取绝对值
 * @returns {string}
 */
var abs = function abs() {
  _AssertUtil2.default.canCast(this, 'bignumber');
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num = new BN(this);
  return num.abs().toString();
};

/**
 * 转换为二进制字符串
 * @returns {string}
 */
var decimalToBinString = function decimalToBinString() {
  _AssertUtil2.default.canCast(this, 'bignumber');
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num = new BN(this);
  return '0b' + num.toString(2);
};

/**
 * 十进制转换为八进制字符串
 * @returns {string}
 */
var decimalToOctString = function decimalToOctString() {
  _AssertUtil2.default.canCast(this, 'bignumber');
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num = new BN(this);
  return '0o' + num.toString(8);
};

/**
 * 十进制转换为十六进制字符串
 * @returns {string}
 */
var decimalToHexString = function decimalToHexString() {
  _AssertUtil2.default.canCast(this, 'bignumber');
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num = new BN(this);
  return '0x' + num.toString(16);
};

/**
 * 直接取整
 * @returns {Number}
 */
var toInt = function toInt() {
  _AssertUtil2.default.canCast(this, 'bignumber');
  return parseInt(this);
};

/**
 * 转换为数值
 * @returns {Number}
 */
var toNumber = function toNumber() {
  _AssertUtil2.default.canCast(this, 'bignumber');
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num = new BN(this);
  return num.toNumber();
};

/**
 * hex string转换为number
 * @returns {number}
 */
var hexToNumber = function hexToNumber() {
  var temp = this;
  if (!temp.startsWith('0x')) {
    temp = '0x' + temp;
  }
  _AssertUtil2.default.canCast(temp, 'bignumber');
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num = new BN(temp);
  return num.toNumber();
};

/**
 * '190' --> 'BE'
 * @returns {string}
 */
var numberStrToHex = function numberStrToHex() {
  var byte = this.toNumber();
  var hexByteMap = '0123456789ABCDEF';
  var str = '';
  str += hexByteMap.charAt(byte >> 4);
  str += hexByteMap.charAt(byte & 0x0f);
  return str;
};

/**
 * 判断此值用于计算时是否具有精度问题
 */
var hasPrecisionIssue = function hasPrecisionIssue() {
  _AssertUtil2.default.canCast(this, 'bignumber');
  return this.gt('9007199254740992');
};

// 字符串处理

/**
 * 移除开头几位字符串
 * @param num
 * @returns {Array}
 */
var removeFirst = function removeFirst(num) {
  return this.substring(num, this.length);
};

/**
 * 移除最后几位字符串
 * @param num
 * @returns {string}
 */
var removeLast = function removeLast(num) {
  return this.substring(0, this.length - num);
};

/**
 * 获取开头几位
 * @param num
 * @returns {string}
 */
var getFirst = function getFirst(num) {
  return this.substring(0, num);
};

/**
 * 获取最后几位
 * @param num
 * @returns {string}
 */
var getLast = function getLast(num) {
  return this.substring(this.length - num, this.length);
};

/**
 * 全部替换(只能简单替换字符串)
 * @param regStr
 * @param replaceStr
 * @returns {string}
 */
var replaceAll = function replaceAll(regStr, replaceStr) {
  return this.replace(new RegExp(regStr, 'gm'), replaceStr);
};

/**
 * 正则查找所有
 * @param regStr
 * @returns {Array}
 */
var findAll = function findAll(regStr) {
  var reg = new RegExp(regStr, 'g');
  reg.compile(reg); //作用是能够对正则表达式进行编译，被编译过的正则在使用的时候效率会更高，适合于对一个正则多次调用的情况下
  var results = [];
  var a = null;
  while (a = reg.exec(this)) {
    //数组中第0个元素是匹配的子字符串，第二个元素是正则中的第一个子分组匹配的结果（如果有子分组，即正则中存在用圆括号括起来的分组），第三个是正则中第二个子分组匹配的结果（如果有第二个子分组）
    results.push({
      outputFull: a[0],
      output: a[1],
      index: a['index'],
      input: a['input']
    });
  }
  return results;
};

/**
 * 从字符串中分类出指定区间的内容
 * @param splitStr1
 * @param splitStr2
 * @returns {{}}
 */
var classify = function classify(splitStr1, splitStr2) {
  var results = [];
  var regStr = splitStr1 + '(.*?)' + splitStr2;
  var targets = this.findAll(regStr);
  for (var i = 0; i < targets.length; i++) {
    if (i === 0) {
      results.push({
        name: this.substring(0, targets[i]['index']),
        is: false,
        index: results.length
      });
    } else {
      results.push({
        name: this.substring(targets[i - 1]['index'] + targets[i - 1]['outputFull'].length, targets[i]['index']),
        is: false,
        index: results.length
      });
    }
    results.push({
      name: targets[i]['output'],
      is: true,
      index: results.length
    });
    if (i === targets.length - 1) {
      results.push({
        name: this.substring(targets[i]['index'] + targets[i]['outputFull'].length, this.length),
        is: false,
        index: results.length
      });
    }
  }
  return results;
};

/**
 * 十六进制字符串转化为Buffer
 * @returns {Array}
 */
var hexToBuffer = function hexToBuffer() {
  var temp = this;
  if (temp.startsWith('0x')) {
    temp = temp.substring(2, temp.length);
  }
  // 长度奇数前面加0
  if (temp.length % 2 !== 0) {
    temp = '0' + temp;
  }
  return Buffer.from ? Buffer.from(temp, 'hex') : new Buffer(temp);
};

/**
 * 普通字符串转buffer
 * @returns {Buffer}
 */
var toBuffer = function toBuffer() {
  return new Buffer(this);
};

/**
 * 十六进制字符串转化为十进制字符串
 * @returns {number|*}
 */
var hexToDecimalString = function hexToDecimalString() {
  var temp = this;
  if (!temp.startsWith('0x')) {
    temp = '0x' + temp;
  }
  _AssertUtil2.default.canCast(temp, 'bignumber');
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num = new BN(temp);
  return num.toString(10);
};

/**
 * 二进制字符串转十进制字符串
 * @returns {number}
 */
var binToDecimalString = function binToDecimalString() {
  var temp = this;
  if (!temp.startsWith('0b')) {
    temp = '0b' + temp;
  }
  _AssertUtil2.default.canCast(temp, 'bignumber');
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num = new BN(temp);
  return num.toString(10);
};

/**
 * 检查是否是严格的hex数据
 * @returns {boolean}
 */
var isStrictHexString = function isStrictHexString() {
  return (/^(-)?0x[0-9a-f]*$/i.test(this)
  );
};

/**
 * 清空hex字符串左边或右边的00
 * @param typeStr {string} left/right/both
 * @returns {string | void | *} 结果不带0x
 */
var clearZeroZero = function clearZeroZero(typeStr) {
  var hex = this.replace(/^0x/i, '');
  if (typeStr === 'left') {
    hex = hex.replace(/^(?:00)*/, '');
  } else if (typeStr === 'right') {
    hex = hex.split('').reverse().join('');
    hex = hex.replace(/^(?:00)*/, '');
    hex = hex.split('').reverse().join('');
  } else if (typeStr === 'both') {
    hex = hex.replace(/^(?:00)*/, '');
    hex = hex.split('').reverse().join('');
    hex = hex.replace(/^(?:00)*/, '');
    hex = hex.split('').reverse().join('');
  }
  return hex;
};

/**
 * 转换成BigNumber对象
 * @returns {BigNumber}
 */
var toBigNumber = function toBigNumber() {
  return new _bignumber2.default(this);
};

/**
 * 移除尾随的0
 */
var removeTrailingZeros = function removeTrailingZeros() {
  var BN = _bignumber2.default.clone({
    EXPONENTIAL_AT: 1e+9
  });
  var num1 = new BN(parseFloat(this));
  return num1.toString();
};

var toArray = function toArray() {
  var len = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
  var arrLen = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;

  if (len !== null && arrLen === null) {
    var num = this.length % len === 0 ? parseInt(this.length / len) : parseInt(this.length / len) + 1;
    var newArrays = [];
    for (var i = 0; i < num; i++) {
      var arr = this.slice(len * i, len * (i + 1));
      if (arr.length > 0) {
        newArrays.push(arr);
      }
    }
    return newArrays;
  } else if (len === null && arrLen !== null) {
    var _newArrays = [];
    var _num = parseInt(this.length / arrLen);
    for (var _i = 0; _i < arrLen; _i++) {
      var _arr = this.slice(_num * _i, _num * (_i + 1));
      if (_arr.length > 0) {
        _newArrays.push(_arr);
      }
    }
    if (_num * arrLen < this.length) {
      _newArrays[_newArrays.length - 1] = _newArrays[_newArrays.length - 1].concat(this.slice(_num * arrLen, this.length));
    }
    return _newArrays;
  } else {
    return null;
  }
};

var hexStrToBase64 = function hexStrToBase64() {
  return Buffer.from(this, 'hex').toString('base64');
};

var base64ToHexStr = function base64ToHexStr() {
  return Buffer.from(this, 'base64').toString('hex');
};

var strToBase64 = function strToBase64() {
  return Buffer.from(this).toString('base64');
};

var base64ToStr = function base64ToStr() {
  return Buffer.from(this, 'base64').toString();
};

var removeLastEnter = function removeLastEnter() {
  if (this.endsWith('\r\n')) {
    return this.removeLast(2);
  }

  if (this.endsWith('\n')) {
    return this.removeLast(1);
  }
  return this;
};

/**
 * 移除字符串最后一段。
 * @param str
 */
var removeLastByStr = function removeLastByStr(str) {
  return this.split(str).removeLastOne().join(str);
};

var removeFirstByStr = function removeFirstByStr(str) {
  return this.split(str).removeFirstOne().join(str);
};

String.prototype.add = add;
String.prototype.sub = sub;
String.prototype.multi = multi;
String.prototype.pow = pow;
String.prototype.div = div;
String.prototype.mod = mod;
String.prototype.gt = gt;
String.prototype.gtOrEq = gtOrEq;
String.prototype.gte = gtOrEq;
String.prototype.lt = lt;
String.prototype.ltOrEq = ltOrEq;
String.prototype.lte = ltOrEq;
String.prototype.eq = eq;
String.prototype.addThousandSign = addThousandSign;
String.prototype.removeThousandSign = removeThousandSign;
String.prototype.remainDecimal = remainDecimal;
String.prototype.decimalCount = decimalCount;
String.prototype.abs = abs;
String.prototype.decimalToBinString = decimalToBinString;
String.prototype.decimalToOctString = decimalToOctString;
String.prototype.decimalToHexString = decimalToHexString;
String.prototype.toInt = toInt;
String.prototype.toNumber = toNumber;
String.prototype.hasPrecisionIssue = hasPrecisionIssue;
String.prototype.removeFirst = removeFirst;
String.prototype.removeLast = removeLast;
String.prototype.replaceAll = replaceAll;
String.prototype.findAll = findAll;
String.prototype.classify = classify;
String.prototype.hexToBuffer = hexToBuffer;
String.prototype.toBuffer = toBuffer;
String.prototype.hexToDecimalString = hexToDecimalString;
String.prototype.binToDecimalString = binToDecimalString;
String.prototype.isStrictHexString = isStrictHexString;
String.prototype.clearZeroZero = clearZeroZero;
String.prototype.toBigNumber = toBigNumber;
String.prototype.shiftedBy = shiftedBy;
String.prototype.unShiftedBy = unShiftedBy;
String.prototype.negated = negated;
String.prototype.removeTrailingZeros = removeTrailingZeros;
String.prototype.toArray = toArray;
String.prototype.hexToNumber = hexToNumber;
String.prototype.numberStrToHex = numberStrToHex;
String.prototype.getFirst = getFirst;
String.prototype.getLast = getLast;
String.prototype.hexStrToBase64 = hexStrToBase64;
String.prototype.strToBase64 = strToBase64;
String.prototype.base64ToHexStr = base64ToHexStr;
String.prototype.base64ToStr = base64ToStr;
String.prototype.sqrt = sqrt;
String.prototype.removeLastEnter = removeLastEnter;
String.prototype.removeLastByStr = removeLastByStr;
String.prototype.removeFirstByStr = removeFirstByStr;