'use strict';

require('../index');

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('stringAssist', function () {

  it('remainDecimal', function () {
    var result = '10.12'.remainDecimal(4, 0);
    _assert2.default.strictEqual(result, '10.1200');
  });

  it('removeFirstByStr', function () {
    var result = '2,4'.removeFirstByStr(',');
    logger.error(result);
    _assert2.default.strictEqual(result, '4');
    var result1 = '2,4,8'.removeFirstByStr(',');
    logger.error(result1);
    _assert2.default.strictEqual(result1, '4,8');
  });

  it('removeLastByStr', function () {
    var result = '2,4'.removeLastByStr(',');
    // logger.error(result)
    _assert2.default.strictEqual(result, '2');
    var result1 = '2,4,8'.removeLastByStr(',');
    // logger.error(result1)
    _assert2.default.strictEqual(result1, '2,4');
  });

  it('removeLastEnter', function () {
    var result = '2746\n'.removeLastEnter();
    // logger.error(result)
    _assert2.default.strictEqual(result, '2746');

    var result1 = '2746\r\n'.removeLastEnter();
    // logger.error(result1)
    _assert2.default.strictEqual(result1, '2746');
  });

  it('pow', function () {
    var result = '2'.pow(0.5);
    logger.error(result);
    _assert2.default.strictEqual(result, '1000000000000000000');
  });

  it('replaceAll', function () {
    var result = '145145abc451abc535'.replaceAll('abc', '111');
    _assert2.default.strictEqual(result, '145145111451111535');
  });

  it('pow', function () {
    var result = '10'.pow(18);
    _assert2.default.strictEqual(result, '1000000000000000000');
  });

  it('add', function () {
    var result = '10'.add('1');
    _assert2.default.strictEqual(result, '11');
  });

  it('sub', function () {
    var result = '10'.sub('1');
    _assert2.default.strictEqual(result, '9');
  });

  it('remainDecimal', function () {
    var result = '10.121'.remainDecimal(0);
    _assert2.default.strictEqual(result, '10');
  });

  it('remainDecimal', function () {
    var result = '10.121565321'.remainDecimal(8, 0);
    _assert2.default.strictEqual(result, '10.12156533');
  });

  it('eq', function () {
    var result = '0x0'.eq(0);
    _assert2.default.strictEqual(result, true);
  });

  it('gt', function () {
    var result = '0.023'.gt('0.0011');
    _assert2.default.strictEqual(result, true);
  });

  it('shiftedBy "18"', function () {
    var result = '1353'.shiftedBy('18');
    _assert2.default.strictEqual(result, '1353000000000000000000');
  });

  it('shiftedBy "-2"', function () {
    var result = '1353'.shiftedBy('-2');
    _assert2.default.strictEqual(result, '13.53');
  });

  it('shiftedBy -3', function () {
    var result = '1353'.shiftedBy(-3);
    _assert2.default.strictEqual(result, '1.353');
  });

  it('shiftedBy 3', function () {
    var result = '1353'.shiftedBy(3);
    _assert2.default.strictEqual(result, '1353000');
  });

  it('utf8HexStringToString', function () {
    var result = '0x3c5554462d383e'.utf8HexStringToString();
    _assert2.default.strictEqual(result, '<UTF-8>');
  });

  it('stringToUtf8HexString', function () {
    var result = '<UTF-8>'.stringToUtf8HexString();
    _assert2.default.strictEqual(result, '0x3c5554462d383e');
  });

  it('toNumber', function () {
    var result = '-2'.toNumber();
    _assert2.default.strictEqual(result, -2);
  });

  it('unShiftedBy', function () {
    var result = '231'.unShiftedBy('2');
    _assert2.default.strictEqual(result, '2.31');
  });

  it('unShiftedBy', function () {
    var result = '231'.unShiftedBy(2);
    _assert2.default.strictEqual(result, '2.31');
  });

  it('negated', function () {
    var result1 = '231'.negated();
    var result2 = '-231'.negated();
    _assert2.default.strictEqual(result1, '-231');
    _assert2.default.strictEqual(result2, '231');
  });

  it('removeTrailingZeros', function () {
    var result1 = '0.004250000'.removeTrailingZeros();
    _assert2.default.strictEqual(result1, '0.00425');

    var result2 = '230.700'.removeTrailingZeros();
    _assert2.default.strictEqual(result2, '230.7');

    var result3 = '230'.removeTrailingZeros();
    _assert2.default.strictEqual(result3, '230');

    var result4 = '0.000000001'.removeTrailingZeros();
    _assert2.default.strictEqual(result4, '0.000000001');
  });

  it('toInt', function () {
    var result1 = '100000000000'.toInt();
    _assert2.default.strictEqual(result1, 100000000000);
  });

  it('toArray', function () {
    var result = '7cce3caf92eb2005ad8a6dd02625d1d8'.toArray(2);
    // logger.error(result)
    _assert2.default.strictEqual(result[0], '7c');
  });

  it('toArray', function () {
    var result = '7cce3caf92eb2005ad8a6dd02625d1d8'.toArray(null, 4);
    // logger.error(result)
    _assert2.default.strictEqual(result[0], '7cce3caf');
  });

  it('hexToNumber', function () {
    var result = '7c'.hexToNumber();
    var result1 = '0x7c'.hexToNumber();
    // logger.error(result)
    _assert2.default.strictEqual(result, 124);
    _assert2.default.strictEqual(result1, 124);
  });

  it('hexToDecimalString', function () {
    var result = '6673064416959ab'.hexToDecimalString();
    var result1 = '0x6673064416959ab'.hexToDecimalString();
    // logger.error(result)
    _assert2.default.strictEqual(result, '461390693981051307');
    _assert2.default.strictEqual(result1, '461390693981051307');
  });

  it('binToDecimalString', function () {
    var result = '11001100111001100000110010001000001011010010101100110101011'.binToDecimalString();
    var result1 = '0b11001100111001100000110010001000001011010010101100110101011'.binToDecimalString();
    // logger.error('test', result)
    _assert2.default.strictEqual(result, '461390693981051307');
    _assert2.default.strictEqual(result1, '461390693981051307');
  });

  it('decimalToBinString', function () {
    var result = '461390693981051307'.decimalToBinString();
    // logger.error(result)
    _assert2.default.strictEqual(result, '0b11001100111001100000110010001000001011010010101100110101011');
  });

  it('decimalToHexString', function () {
    var result = '461390693981051307'.decimalToHexString();
    // logger.error(result)
    _assert2.default.strictEqual(result, '0x6673064416959ab');
  });

  it('decimalToOctString', function () {
    var result = '461390693981051307'.decimalToOctString();
    // logger.error(result)
    _assert2.default.strictEqual(result, '0o31471406210132254653');
  });

  it('decimalToBinString', function () {
    var result = '461390693981051307'.decimalToBinString();
    // logger.error(result)
    _assert2.default.strictEqual(result, '0b11001100111001100000110010001000001011010010101100110101011');
  });

  it('urlEncode', function () {
    var result = '分叉'.urlEncode();
    // logger.error(result)
    _assert2.default.strictEqual(result, '%E5%88%86%E5%8F%89');
  });

  it('urlDecode', function () {
    var result = '%E5%88%86%E5%8F%89'.urlDecode();
    // logger.error(result)
    _assert2.default.strictEqual(result, '分叉');
  });

  it('getFirst', function () {
    var result = 'th356hfgh3httr'.getFirst(4);
    // logger.error(result)
    _assert2.default.strictEqual(result, 'th35');
  });

  it('getLast', function () {
    var result = 'th356hfgh3httr'.getLast(4);
    // logger.error(result)
    _assert2.default.strictEqual(result, 'httr');
  });

  // numberStrToHex
  it('numberStrToHex', function () {
    var result = '190'.numberStrToHex();
    // logger.error(result)
    _assert2.default.strictEqual(result, 'BE');
  });

  it('hexStrToBase64', function () {
    var result = '0c68affbcf01a49540e590bcf88afa4dc630eb5ce0e13a5b067c918f0e1c9af4'.hexStrToBase64();
    // logger.error(result)
    // MGM2OGFmZmJjZjAxYTQ5NTQwZTU5MGJjZjg4YWZhNGRjNjMwZWI1Y2UwZTEzYTViMDY3YzkxOGYwZTFjOWFmNA==
    _assert2.default.strictEqual(result, 'DGiv+88BpJVA5ZC8+Ir6TcYw61zg4TpbBnyRjw4cmvQ=');
  });

  it('strToBase64', function () {
    var result = '0c68affbcf01a49540e590bcf88afa4dc630eb5ce0e13a5b067c918f0e1c9af4'.strToBase64();
    // logger.error(result)
    _assert2.default.strictEqual(result, 'MGM2OGFmZmJjZjAxYTQ5NTQwZTU5MGJjZjg4YWZhNGRjNjMwZWI1Y2UwZTEzYTViMDY3YzkxOGYwZTFjOWFmNA==');
  });

  it('strToBase64', function () {
    var result = 'www.baidu.com'.strToBase64();
    logger.error(result);
    // assert.strictEqual(result, 'MGM2OGFmZmJjZjAxYTQ5NTQwZTU5MGJjZjg4YWZhNGRjNjMwZWI1Y2UwZTEzYTViMDY3YzkxOGYwZTFjOWFmNA==')
  });

  it('base64ToHexStr', function () {
    var result = 'DGiv+88BpJVA5ZC8+Ir6TcYw61zg4TpbBnyRjw4cmvQ='.base64ToHexStr();
    // logger.error(result)
    _assert2.default.strictEqual(result, '0c68affbcf01a49540e590bcf88afa4dc630eb5ce0e13a5b067c918f0e1c9af4');
  });

  it('base64ToStr', function () {
    var result = 'MGM2OGFmZmJjZjAxYTQ5NTQwZTU5MGJjZjg4YWZhNGRjNjMwZWI1Y2UwZTEzYTViMDY3YzkxOGYwZTFjOWFmNA=='.base64ToStr();
    // logger.error(result)
    _assert2.default.strictEqual(result, '0c68affbcf01a49540e590bcf88afa4dc630eb5ce0e13a5b067c918f0e1c9af4');
  });
});